function hc = meddis(bm,fs)

% hc = meddis(bm,fs)
% Applies Ray Meddis's IHC model to input, which is assumed to be the
% result of peripheral filtering. Note the need for correct normalisation
% prior to this routine in order that the hair cell is in the operating range
% for signals around 30-60 dB SPL.
%
% Inputs:
% bm - output of basilar membrane simulation (e.g. gammatone filter)
% fs - sampling frequency
%
% Output:
% hc - firing probability
%
% Guy Brown, University of Sheffield
% Based on the BASIC code in Ray's 1990 JASA paper
%
% This software is provided "as is" without warranty of any kind

MED_Y=8;
MED_G=2000.0;
MED_L=2500.0;
MED_R=6580.0;
MED_X=66.31;
MED_A=2.0;
MED_B=300.0;
MED_H=180E6;
MED_M=1.0;

dt=1/fs;
ymdt=MED_Y*MED_M*dt;
xdt=MED_X*dt;
ydt=MED_Y*dt;
lplusrdt=(MED_L+MED_R)*dt;
rdt=MED_R*dt;
gdt=MED_G*dt;
hdt=MED_H*dt;
kt=MED_G*MED_A/(MED_A+MED_B);
c=MED_M*MED_Y*kt/(MED_L*kt+MED_Y*(MED_L+MED_R));
q=c*(MED_L+MED_R)/kt;
w=c*MED_R/MED_X;

len=length(bm);
hc=zeros(1,len);

for t=1:len
    x=bm(t)+MED_A;
    if (x > 0.0)
        kt=gdt*(x/(x+MED_B));
    else
        kt=0.0;
    end
    if (q < MED_M )
        replenish=ymdt-ydt*q;
    else
        replenish=0.0;
    end
    eject=kt*q;
    reuptakeandloss=lplusrdt*c;
    reuptake=rdt*c;
    reprocess=xdt*w;
    q = max(0.0,q+replenish-eject+reprocess);
    c = max(0.0,c+eject-reuptakeandloss);
    w = max(0.0,w+reuptake-reprocess);
    hc(t) = hdt*c;
end

